import json
import time

from medsenger_api import AgentApiClient, prepare_file
from unittest import TestCase
from tests.config import *


class TestApi(TestCase):
    grpc_client = AgentApiClient(API_KEY, MAIN_HOST, AGENT_ID, True, True, 'localhost')

    def test_get_records_from_multiple_categories(self):
        for i in range(1):

            S = time.time()
            G = self.grpc_client.get_records(CONTRACT_ID, "systolic_pressure,diastolic_pressure,pulse")
            print("G time:", time.time() - S)


from medsenger_api import AgentApiClient, prepare_file
from unittest import TestCase
from tests.config import *

class TestApi(TestCase):

    def create_client(self):
        return AgentApiClient(API_KEY, MAIN_HOST, AGENT_ID, True)

    def test_patient_info(self):
        print(self.create_client().get_patient_info(CONTRACT_ID))

    def test_get_categories(self):
        print(self.create_client().get_categories())

    def test_get_available_categories(self):
        print(self.create_client().get_available_categories(CONTRACT_ID))

    def test_get_records(self):
        print(self.create_client().get_records(CONTRACT_ID, "systolic_pressure"))

    def test_get_records_from_multiple_categroies(self):
        records = self.create_client().get_records(CONTRACT_ID, "systolic_pressure,diastolic_pressure", inner_list=True, group=False)

        for record in records:
            print(record['category_info']['name'], record.get('group'))

    def test_send_message_with_attachments(self):
        client = self.create_client()
        client.send_message(contract_id=CONTRACT_ID, text='', attachments=[prepare_file('.gitignore')], send_from='patient')

    def test_outdate_message(self):
        client = self.create_client()
        messsage = client.send_message(contract_id=CONTRACT_ID, text='', attachments=[prepare_file('.gitignore')],
                            send_from='patient')

        client.outdate_message(CONTRACT_ID, messsage['id'])



    def test_add_record_with_file(self):
        client = self.create_client()

        result = client.add_record(CONTRACT_ID, 'systolic_pressure', value=120, files=[prepare_file('test_api.py')])
        print(result)

    def test_get_file(self):
        client = self.create_client()

        result = client.get_file(CONTRACT_ID, 4)

        print(result)

    def test_clinics_info(self):
        client = self.create_client()

        print(client.get_clinics_info())

    def test_add_record_with_id_returns_ids(self):
        client = self.create_client()

        result = client.add_record(CONTRACT_ID, 'systolic_pressure', value=120, return_id=True)

        print(result)

        id = result[0]

        print(client.get_record_by_id(CONTRACT_ID, id))

    def test_delete_record(self):
        client = self.create_client()

        result = client.add_record(CONTRACT_ID, 'systolic_pressure', value=120, return_id=True)
        id = result[0]
        client.delete_record(CONTRACT_ID, id)

        assert client.get_record_by_id(CONTRACT_ID, id) == None

    def test_get_messsages(self):
        client = self.create_client()

        print(client.get_messages(CONTRACT_ID))

    def test_set_classifier(self):
        client = self.create_client()

        client.set_classifier(CONTRACT_ID, "unstandart")

    def test_set_contract_param(self):
        client = self.create_client()

        client.set_contract_param(CONTRACT_ID, "param", "value")

    def test_remove_contract_param(self):
        client = self.create_client()

        client.set_contract_param(CONTRACT_ID, "param", "")




import time
from .utils import gts
from .grpc_client import RecordsClient
from .rest_client import RestApiClient

try:
    import sentry_sdk
except:
    pass


class AgentApiClient:
    def __init__(self, api_key, host="https://medsenger.ru", agent_id=None, debug=False, use_grpc=False,
                 grpc_host=None, sentry_dsn=None, locale=None):

        self.rest_client = RestApiClient(api_key, host, agent_id, debug, locale=locale)
        self.grpc_client = None
        self.user_cache = {}
        self.categories_cache = {}
        self.debug = debug
        self.dsn = sentry_dsn
        self.locale = locale

        self.categories = []
        self.categories_last_request = 0

        if use_grpc:
            self.grpc_client = RecordsClient(host=grpc_host, debug=debug, api_key=api_key, locale=locale)

        if self.dsn:
            sentry_sdk.init(dsn=self.dsn, traces_sample_rate=1.0)

    def set_locale(self, locale):
        self.locale = locale
        self.rest_client.locale = locale
        if self.grpc_client:
            self.grpc_client.locale = locale

    def get_categories(self, forced_locale=None):
        if self.categories and time.time() - self.categories_last_request < 60:
            return self.categories

        self.categories_last_request = time.time()

        if self.grpc_client:
            try:
                return self.grpc_client.get_categories(forced_locale=forced_locale)
            except Exception as e:
                if self.dsn:
                    sentry_sdk.capture_exception(e)
                print(gts(), "GRPC failed with error:", e)

        self.categories = self.rest_client.get_categories()

        if self.locale or forced_locale:
            locale = forced_locale
            if not locale:
                locale = self.locale
            self.categories = [category for category in self.categories if category['locale'] == locale]

        return self.categories

    def reconnect(self):
        if self.grpc_client:
            self.grpc_client.reconnect()

    def get_available_categories(self, contract_id, forced_locale=None):
        if self.grpc_client:
            try:
                if contract_id not in self.user_cache:
                    self.get_patient_info(contract_id)

                if contract_id in self.user_cache:
                    return self.grpc_client.get_categories_for_user(self.user_cache[contract_id], forced_locale=forced_locale)

                return []
            except Exception as e:
                if self.dsn:
                    sentry_sdk.capture_exception(e)
                print(gts(), "GRPC failed with error:", e)

        categories = self.rest_client.get_available_categories(contract_id)

        if self.locale or forced_locale:
            locale = forced_locale
            if not locale:
                locale = self.locale
            categories = [category for category in categories if category['locale'] == locale]

        return categories

    def get_patient_info(self, contract_id):
        result = self.rest_client.get_patient_info(contract_id)

        if self.debug:
            print(result)

        if result.get('id'):
            self.user_cache[contract_id] = result.get('id')

        return result

    def get_clinics_info(self):
        return self.rest_client.get_clinics_info()

    def __prepare_query_for_grpc(self, contract_id, category_name=None, time_from=None, time_to=None, limit=None,
                                 offset=None, group=False, inner_list=False, user_id=None):

        print("Preparing q for gprc request in api client:", (contract_id, category_name, time_from, time_to, limit, offset, group, inner_list, user_id))
        if contract_id not in self.user_cache and not user_id:
            self.get_patient_info(contract_id)

        if not user_id:
            user_id = self.user_cache[contract_id]

        return dict(user_id=user_id, category_name=category_name, time_from=time_from, time_to=time_to, offset=offset, limit=limit, group=group, inner_list=inner_list)

    def get_multiple_records(self, queries, forced_locale=None):
        if self.grpc_client:
            try:
                prepared_queries = [self.__prepare_query_for_grpc(**query) for query in queries]
                return self.grpc_client.get_multiple_records(prepared_queries, forced_locale=forced_locale)
            except Exception as e:
                if self.dsn:
                    sentry_sdk.capture_exception(e)
                print(gts(), "GRPC for multiple records failed with error:", e)

        return [self.get_records(**query, forced_locale=forced_locale) for query in queries]

    def get_records(self, contract_id, category_name=None, time_from=None, time_to=None, limit=None, offset=None,
                    group=False, return_count=False, inner_list=False, user_id=None, forced_locale=None):

        if self.grpc_client:
            try:
                if return_count:
                    method = self.grpc_client.count_records
                else:
                    method = self.grpc_client.get_records

                return method(
                    **self.__prepare_query_for_grpc(contract_id, category_name, time_from, time_to, limit, offset, group,
                                                    inner_list, user_id), forced_locale=forced_locale)
            except Exception as e:
                if self.dsn:
                    sentry_sdk.capture_exception(e)
                print(gts(), "GRPC failed with error:", e)

        return self.rest_client.get_records(contract_id, category_name, time_from, time_to, limit, offset, group,
                                            return_count, inner_list, forced_locale)

    def get_record_by_id(self, contract_id, record_id, forced_locale=None):
        if self.grpc_client:
            try:
                return self.grpc_client.get_record_by_id(record_id, forced_locale=forced_locale)
            except Exception as e:
                if self.dsn:
                    sentry_sdk.capture_exception(e)
                print(gts(), "GRPC failed with error:", e)

        return self.rest_client.get_record_by_id(contract_id, record_id)

    def add_hooks(self, contract_id, names):
        return self.rest_client.add_hooks(contract_id, names)

    def set_classifier(self, contract_id, code):
        return self.rest_client.set_classifier(contract_id, code)

    def remove_hooks(self, contract_id, names):
        return self.rest_client.remove_hooks(contract_id, names)

    def send_addition(self, contract_id, record_id, addition):
        return self.rest_client.send_addition(contract_id, record_id, addition)

    def add_record(self, contract_id, category_name, value, record_time=None, params=None, files=None, return_id=False,
                   replace=False):
        return self.rest_client.add_record(contract_id, category_name, value, record_time, params, files, return_id,
                                           replace)

    def delete_record(self, contract_id, record_id):
        return self.rest_client.delete_record(contract_id, record_id)

    def add_records(self, contract_id, values, record_time=None, params={}, return_id=False):
        return self.rest_client.add_records(contract_id, values, record_time, params, return_id)

    def send_message(self, contract_id, text, action_link=None, action_name=None, action_onetime=True,
                     only_doctor=False,
                     only_patient=False, action_deadline=None, is_urgent=False, need_answer=False,
                     attachments=None, action_big=True, send_from=None, forward_to_doctor=True, action_type='action'):
        return self.rest_client.send_message(contract_id, text, action_link, action_name, action_onetime,
                                             only_doctor,
                                             only_patient, action_deadline, is_urgent, need_answer,
                                             attachments, action_big, send_from, forward_to_doctor, action_type)

    def outdate_message(self, contract_id, message_id):
        return self.rest_client.outdate_message(contract_id, message_id)

    def finish_task(self, contract_id, task_id):
        return self.rest_client.finish_task(contract_id, task_id)

    def delete_task(self, contract_id, task_id):
        return self.rest_client.delete_task(contract_id, task_id)

    def add_task(self, contract_id, text, target_number=1, date=None, important=False, action_link=None):
        return self.rest_client.add_task(contract_id, text, target_number, date, important, action_link)

    def request_payment(self, contract_id, inv_id, amount, title):
        return self.rest_client.request_payment(contract_id, inv_id, amount, title)

    def get_payments(self, contract_id):
        return self.rest_client.get_payments(contract_id)

    def send_order(self, contract_id, order, receiver_id=None, params=None):
        return self.rest_client.send_order(contract_id, order, receiver_id, params)

    def get_agent_token(self, contract_id):
        return self.rest_client.get_agent_token(contract_id)

    def download_file(self, *args, **kwargs):
        return self.get_file(*args, **kwargs)

    def download_attachment(self, *args, **kwargs):
        return self.get_attachment(*args, **kwargs)

    def download_image(self, *args, **kwargs):
        return self.get_image(*args, **kwargs)

    def get_file(self, contract_id, file_id):
        return self.rest_client.get_file(contract_id, file_id)

    def get_attachment(self, attachment_id):
        return self.rest_client.get_attachment(attachment_id)

    def get_image(self, image_id, size):
        return self.rest_client.get_image(image_id, size)

    def update_cache(self, contract_id):
        return self.rest_client.update_cache(contract_id)

    def set_info_materials(self, contract_id, materials):
        return self.rest_client.set_info_materials(contract_id, materials)

    def set_contract_param(self, contract_id, name, value):
        return self.rest_client.set_contract_param(contract_id, name, value)

    def ajax_url(self, action, contract_id, agent_token):
        # TODO fix
        return self.host.replace('8001',
                                 '8000') + "/api/client/agents/{agent_id}/?action={action}&contract_id={contract_id}&agent_token={agent_token}".format(
            agent_id=self.agent_id, action=action, contract_id=contract_id, agent_token=agent_token
        )

    def get_messages(self, contract_id, from_id=0):
        return self.rest_client.get_messages(contract_id, from_id)

    def notify_admin(self, message, channel="it"):
        return self.rest_client.notify_admin(message, channel)


import json
import time

from medsenger_api import AgentApiClient, prepare_file
from unittest import TestCase
from tests.config import *


class TestApi(TestCase):
    grpc_client = AgentApiClient(API_KEY, MAIN_HOST, AGENT_ID, True, True, 'localhost', sentry_dsn=DSN)
    default_client = AgentApiClient(API_KEY, MAIN_HOST, AGENT_ID, True, False)

    def test_get_categories(self):
        assert self.grpc_client.get_categories() == self.default_client.get_categories()

    def test_get_available_categories(self):
        G = self.grpc_client.get_available_categories(CONTRACT_ID)
        D = self.default_client.get_available_categories(CONTRACT_ID)

        for a, b in zip(G, D):
            if a != b:
                print("a: ", a)
                print("b: ", b)

        assert G == D



    def test_get_records(self):

        S = time.time()
        G = self.grpc_client.get_records(CONTRACT_ID, "systolic_pressure")
        print("G time:", time.time() - S)

        S = time.time()
        D = self.default_client.get_records(CONTRACT_ID, "systolic_pressure")
        print("D time:", time.time() - S)

        for a, b in zip(G['values'], D['values']):
            if a != b:
                print("a: ", a)
                print("b: ", b)

        assert G == D

    def test_get_records_with_group(self):
        S = time.time()
        G = self.grpc_client.get_records(CONTRACT_ID, "systolic_pressure,pulse", limit=3, group=True)
        print("G time:", time.time() - S)

        S = time.time()
        D = self.default_client.get_records(CONTRACT_ID, "systolic_pressure,pulse", limit=3, group=True)
        print("D time:", time.time() - S)

        print(G, '\n\n', D)

        assert G == D

    def test_get_records_from_multiple_categories(self):
        S = time.time()
        G = self.grpc_client.get_records(CONTRACT_ID, "systolic_pressure,diastolic_pressure,pulse", limit=20)
        print("G time:", time.time() - S)

        S = time.time()
        D = self.default_client.get_records(CONTRACT_ID, "systolic_pressure,diastolic_pressure,pulse", limit=20)
        print("D time:", time.time() - S)

        print("records count:", len(D))

        assert G == D

    def test_get_records_from_multiple_categroies_with_offset(self):
        S = time.time()
        G = self.grpc_client.get_records(CONTRACT_ID, "systolic_pressure,diastolic_pressure,pulse", offset=100, limit=100)
        print("G time:", time.time() - S)

        S = time.time()
        D = self.default_client.get_records(CONTRACT_ID, "systolic_pressure,diastolic_pressure,pulse", offset=100, limit=100)
        print("D time:", time.time() - S)


        print(type(G), type(D))

        for a, b in zip(G, D):
            if a != b:
                print("a: ", a)
                print("b: ", b)

        assert G == D

    def test_get_records_from_multiple_categories_with_time(self):
        T = int(time.time() - 2 * 30 * 24 * 60 * 60)
        F = int(time.time() - 3 * 30 * 24 * 60 * 60)

        S = time.time()
        G = self.grpc_client.get_records(CONTRACT_ID, "systolic_pressure,diastolic_pressure,pulse", time_from=F, time_to=T)
        print("G time:", time.time() - S)

        S = time.time()
        D = self.default_client.get_records(CONTRACT_ID, "systolic_pressure,diastolic_pressure,pulse", time_from=F, time_to=T)
        print("D time:", time.time() - S)


        print(type(G), type(D))

        if D and G:
            for a, b in zip(G, D):
                if a != b:
                    print("a: ", a)
                    print("b: ", b)

        assert G == D

    def test_multiple_get_records(self):
        T = int(time.time() - 2 * 30 * 24 * 60 * 60)
        F = int(time.time() - 3 * 30 * 24 * 60 * 60)

        query_1 = dict(contract_id=CONTRACT_ID, category_name="systolic_pressure,diastolic_pressure,pulse")
        query_2 = dict(contract_id=CONTRACT_ID, category_name="pulse")

        G = self.grpc_client.get_multiple_records([query_1, query_2])

        print(G)

    def test_count_records(self):
        S = time.time()
        G = self.grpc_client.get_records(CONTRACT_ID, "systolic_pressure,diastolic_pressure,pulse", return_count=True)
        print("G time:", time.time() - S)

        S = time.time()
        D = self.default_client.get_records(CONTRACT_ID, "systolic_pressure,diastolic_pressure,pulse", return_count=True)
        print("D time:", time.time() - S)

        assert G == D

    def test_get_single_record(self):
        record = self.default_client.add_record(CONTRACT_ID, "pulse", 60, return_id=True)

        D = self.default_client.get_record_by_id(CONTRACT_ID, record[0])
        G = self.grpc_client.get_record_by_id(CONTRACT_ID, record[0])

        print(D, G)

        assert D == G

    def test_get_single_record_404(self):
        record = self.default_client.add_record(CONTRACT_ID, "pulse", 60, return_id=True)

        D = self.default_client.get_record_by_id(CONTRACT_ID, record[0] + 1)
        G = self.grpc_client.get_record_by_id(CONTRACT_ID, record[0] + 1)

        print(D, G)

        assert D == G


from medsenger_api import prepare_file, prepare_binary
from unittest import TestCase

class TestFileHelpers(TestCase):
    def test_prepare_file(self):
        result = prepare_file("test_file_helpers.py")
        self.assertEqual(result['type'], 'text/x-script.python')

    def test_prepare_binary(self):
        result = prepare_binary("test_file_helpers.py", open("test_file_helpers.py", "rb").read())

        self.assertEqual(result['type'], 'text/x-script.python')


import json
import time
import grpc
from .utils import *
from .protocol import records_pb2_grpc as pb2_grpc
from .protocol import records_pb2 as pb2
from .certificates import CERTTIFICATE


class RecordsClient(object):
    def __get_method__(self, method, stub):
        if method == 'GetCategoryList':
            return stub.GetCategoryList
        elif method == 'GetCategoryListForUser':
            return stub.GetCategoryListForUser
        elif method == 'GetRecordById':
            return stub.GetRecordById
        elif method == 'GetRecords':
            return stub.GetRecords
        elif method == 'CountRecords':
            return stub.CountRecords
        elif method == 'GetMultipleRecords':
            return stub.GetMultipleRecords
        else:
            return None

    def __create_connection__(self):
        if self.channel:
            if self.__debug:
                print("Closing GRPC connection")

            self.channel.close()

        if self.__debug:
            print("Connecting to GRPC...")

        credentials = grpc.ssl_channel_credentials(root_certificates=CERTTIFICATE)

        # instantiate a channel
        self.channel = grpc.secure_channel(
            '{}:{}'.format(self.host, self.server_port), credentials, options=[
                ('grpc.max_send_message_length', 50 * 1024 * 1024),
                ('grpc.max_receive_message_length', 50 * 1024 * 1024),
                ('grpc.keepalive_time_ms', 10000),
                ('grpc.keepalive_timeout_ms', 5000),
                ('grpc.keepalive_permit_without_calls', True),
                ('grpc.http2.max_pings_without_data', 0),
                ('grpc.http2.min_time_between_pings_ms', 10000),
                ('grpc.http2.min_ping_interval_without_data_ms', 5000)
            ])

        # bind the client and the server
        self.stub = pb2_grpc.RecordsStub(self.channel)

        if self.__debug:
            print("Connected to GRPC")

    def reconnect(self):
        self.__create_connection__()

    def __connect_if_should(self):
        if not self.channel:
            self.__create_connection__()

    def __close_connection__(self):
        self.channel.close()

    def __init__(self, debug=False, host=None, api_key=None, locale=None):
        if not host:
            host = "medsenger.ru"

        self.host = host
        self.server_port = 50051
        self.__categories_by_id = {}
        self.__categories_by_name = {}

        self.__debug = debug
        self.channel = None
        self.api_key = api_key
        self.locale = locale

    def __find_category_by_id(self, id, forced_locale=None):
        if not self.__categories_by_id or id not in self.__categories_by_id:
            print("Loading categories")
            self.get_categories()

        categories = self.__categories_by_id.get(id)

        if not categories:
            return None

        if self.locale or forced_locale:
            locale = forced_locale
            if not locale:
                locale = self.locale

            result = [category for category in categories if category.locale == locale]
            if result:
                return result[0]

        return categories[0]

    def __find_category_by_name(self, name, forced_locale=None):
        if not self.__categories_by_name or name not in self.__categories_by_name:
            print("Loading categories")
            self.get_categories()

        categories = self.__categories_by_name.get(name)

        if not categories:
            return None

        if self.locale or forced_locale:
            locale = forced_locale
            if not locale:
                locale = self.locale

            result = [category for category in categories if category.locale == locale]
            if result:
                return result[0]

        return categories[0]

    def __find_ids_for_categories(self, category_names):
        if not self.__categories_by_name:
            print("Loading categories")
            self.get_categories()

        result = set()

        for name in category_names:
            if name not in self.__categories_by_name:
                continue
            for category in self.__categories_by_name[name]:
                result.add(category.id)

        return list(result)

    def __present_category(self, category):
        return {
            "id": category.id,
            "name": category.name,
            "description": category.description,
            "unit": category.unit,
            "type": category.type,
            "default_representation": category.default_representation,
            "is_legacy": category.is_legacy,
            "doctor_can_add": category.doctor_can_add,
            "doctor_can_replace": category.doctor_can_replace,
            "subcategory": category.subcategory if category.subcategory else None,
            "locale": category.locale if category.locale else None
        }

    def __present_file(self, file):
        return {
            "id": file.id,
            "name": file.name,
            "type": file.type,
        }

    def __convert_value(self, value, category):
        if category.type == 'integer':
            return int(float(value.replace(',', '.')))
        if category.type == 'float':
            return round(float(value.replace(',', '.')), 3)
        return value

    def __present_record(self, record, with_category=False, forced_locale=None):
        category = self.__find_category_by_id(record.category_id, forced_locale)

        presentation = {
            "id": record.id,
            "value": self.__convert_value(record.value, category),
            "timestamp": record.created_at,
            "uploaded": record.updated_at,
            "source": {
                "id": record.source.id if record.source.id else None,
                "name": record.source.name
            },
            "group": record.group if record.group else None,
            "params": json.loads(record.params),
            "additions": json.loads(record.additions),
            "attached_files": [self.__present_file(file) for file in record.attached_files],
        }

        if with_category:
            presentation['category_info'] = self.__present_category(category)

        return presentation

    def __make_request(self, method, *args, timeout=20, tries=3):
        self.__connect_if_should()

        if self.__debug:
            print(gts(), " GRPC request to {} with params {} and locale {}".format(method, args, self.locale))

        exception = None
        for i in range(tries):
            try:
                F = self.__get_method__(method, self.stub)

                if F:
                    return F(*args, timeout=timeout, metadata=[('authorization', self.api_key)])
                else:
                    raise Exception("Wrong method")
            except grpc.RpcError as e:
                print("Exception in GRPC request: ", e)

                if e.code() == grpc.StatusCode.NOT_FOUND:
                    return None
                else:
                    exception = e

                    time.sleep(tries)

                    self.__create_connection__()

                    if self.__debug:
                        print("Retrying request... {}".format(i + 1))

        print("Request failed!")
        raise GrpcConnectionError("GRPC request error to {} with params {}: {}".format(method, args, exception))

    @safe
    def get_categories(self, forced_locale=None):
        request = pb2.Empty()

        result = self.__make_request('GetCategoryList', request)

        categories = []

        for category in result.categories:
            if category.id not in self.__categories_by_id:
                self.__categories_by_id[category.id] = []
            if category.name not in self.__categories_by_name:
                self.__categories_by_name[category.name] = []

            self.__categories_by_id[category.id].append(category)
            self.__categories_by_name[category.name].append(category)

            categories.append(self.__present_category(category))

        if self.locale or forced_locale:
            locale = forced_locale
            if not locale:
                locale = self.locale

            print("Filtering categories by locale:", locale)
            categories = [category for category in categories if category['locale'] == locale]

        return categories

    @safe
    def get_categories_for_user(self, user_id, forced_locale=None):
        locale = forced_locale
        if not locale and self.locale:
            locale = self.locale

        request = pb2.User(id=user_id)

        result = self.__make_request('GetCategoryListForUser', request)

        categories = []
        for category in result.categories:
            if locale and category.locale != locale:
                continue
            categories.append(self.__present_category(category))
        return categories

    @safe
    def get_record_by_id(self, record_id, forced_locale=None):
        request = pb2.RecordRequest(id=record_id)

        record = self.__make_request('GetRecordById', request)

        if record:
            return self.__present_record(record, forced_locale=forced_locale)
        else:
            return None

    def __prepare_record_query(self, user_id, category_name, time_from, time_to, offset, limit, group,
                               inner_list=False, forced_locale=None):
        category_names = [category_name.strip() for category_name in category_name.split(',') if category_name]

        category_ids = self.__find_ids_for_categories(category_names)

        if inner_list:
            category_ids.append(-1)

        if category_names and not category_ids:
            if self.__debug:
                print("Category search failed:", category_names, self.__categories_by_name.keys())

            return None, 0

        if time_from is not None:
            time_from = int(time_from)

        if time_to is not None:
            time_to = int(time_to)

        if offset is not None:
            offset = int(offset)

        if limit is not None:
            limit = int(limit)

        return pb2.RecordQuery(user_id=user_id, category_ids=category_ids, from_timestamp=time_from,
                               to_timestamp=time_to, offset=offset, limit=limit, with_group=group)

    def __present_record_query_answer(self, answer, category_name, full_list=False, forced_locale=None):
        records = None
        if full_list:
            if answer.records:
                records = [self.__present_record(record, with_category=True, forced_locale=forced_locale) for record in answer.records]
        else:
            if answer.records:
                records = {
                    "category": self.__present_category(self.__find_category_by_name(category_name, forced_locale)),
                    "values": [self.__present_record(record, with_category=False, forced_locale=forced_locale) for record in answer.records]
                }

        return records, answer.count

    def __aggregate_records(self, method, user_id, category_name, time_from=0, time_to=int(time.time()),
                            offset=0,
                            limit=None, group=False, inner_list=False, forced_locale=None):
        full_list = not category_name or ',' in category_name or inner_list

        request = self.__prepare_record_query(user_id, category_name, time_from, time_to, offset, limit, group)
        result = self.__make_request(method, request)

        return self.__present_record_query_answer(result, category_name, full_list, forced_locale=forced_locale)

    def get_records(self, user_id, category_name, time_from=0, time_to=int(time.time()), offset=0,
                    limit=None, group=False, inner_list=False, forced_locale=None):

        records, count = self.__aggregate_records('GetRecords', user_id, category_name, time_from,
                                                  time_to,
                                                  offset, limit, group, inner_list, forced_locale=forced_locale)
        print(records, count)
        return records

    def get_multiple_records(self, queries, forced_locale=None):
        prepared_queries = [self.__prepare_record_query(**query) for query in queries]
        request = pb2.MultiRecordQuery(queries=prepared_queries)

        result = self.__make_request('GetMultipleRecords', request)
        answers = []
        for i in range(len(queries)):
            category_name = queries[i]['category_name']
            full_list = not category_name or ',' in category_name or queries[i].get('inner_list', False)
            records, count = self.__present_record_query_answer(result.answers[i], category_name, full_list, forced_locale=forced_locale)
            answers.append(records)

        return answers

    def count_records(self, user_id, category_name, time_from=0, time_to=int(time.time()), offset=0,
                      limit=None, group=False, inner_list=False, forced_locale=None):

        records, count = self.__aggregate_records('CountRecords', user_id, category_name, time_from,
                                                  time_to,
                                                  offset, limit, group, inner_list, forced_locale=forced_locale)

        return {"count": count}